#!/bin/bash

set -e

# Configuration
LIBGTASRC="$HOME/src/gta/sources/gta/libgta"

function build_package ()
{
CROSSSHORTNAME="$1"
CROSSPREFIX="$2"

unset CPPFLAGS
unset CFLAGS
unset CXXFLAGS
unset LDFLAGS
D="`pwd`"
TMPD="`mktemp -d`"
cd "$TMPD"

tar xjvf "$D/zlib-1.2.5.tar.bz2"
cd zlib-1.2.5
make -f win32/Makefile.gcc PREFIX=$CROSSPREFIX- SHARED_MODE=1
make install -f win32/Makefile.gcc INCLUDE_PATH="$TMPD/install/include" LIBRARY_PATH="$TMPD/install/lib" BINARY_PATH="$TMPD/install/bin" SHARED_MODE=1
cd ..
mv install/lib/libzdll.a install/lib/libz.dll.a
ln -s ../bin/zlib1.dll install/lib/libz.dll

tar xzvf "$D/bzip2-1.0.6.tar.gz"
cd bzip2-1.0.6
sed -i 's,sys\\stat\.h,sys/stat.h,g' bzip2.c
sed -i 's,#ifdef _WIN32,#if 0,g' bzlib.h
make libbz2.a PREFIX="$TMPD/install" CC=$CROSSPREFIX-gcc AR=$CROSSPREFIX-ar RANLIB=$CROSSPREFIX-ranlib
$CROSSPREFIX-dlltool --export-all-symbols -D libbz2.dll -e exports.o -l libbz2.dll.a blocksort.o huffman.o crctable.o randtable.o compress.o decompress.o bzlib.o
$CROSSPREFIX-gcc -shared -o libbz2.dll blocksort.o huffman.o crctable.o randtable.o compress.o decompress.o bzlib.o exports.o
install -d "$TMPD/install/include"
install -m664 bzlib.h "$TMPD/install/include"
install -d "$TMPD/install/lib"
install -m664 libbz2.a "$TMPD/install/lib"
install -m664 libbz2.dll.a "$TMPD/install/lib"
install -d "$TMPD/install/bin"
install -m664 libbz2.dll "$TMPD/install/bin"
cd ..
ln -s ../bin/libbz2.dll install/lib/libbz2.dll

tar xzvf "$D/xz-5.0.3.tar.gz"
cd xz-5.0.3
./configure --host=$CROSSPREFIX --prefix="$TMPD/install" --disable-threads --disable-nls --disable-rpath
make -C src/liblzma install-strip
cd ../install/bin
LIBLZMADLL="`ls -1 liblzma-*.dll`"
cd ../..
ln -s ../bin/"$LIBLZMADLL" install/lib/liblzma.dll
rm install/lib/liblzma.la
rm -r install/lib/pkgconfig
rm -r install/share

# Build
mkdir build
cd build
# 1. Only the DLLs
"$LIBGTASRC"/configure --host=$CROSSPREFIX --prefix="$TMPD/install" CFLAGS="-W -Wall -O2" \
	--docdir="$TMPD"/install/doc --disable-reference --disable-static --disable-rpath --disable-la-file
make
make install-strip
# 2. Only the static libs
"$LIBGTASRC"/configure --host=$CROSSPREFIX --prefix="$TMPD"/install CFLAGS="-W -Wall -O2" \
	--docdir="$TMPD"/install/doc --disable-shared --disable-rpath --disable-la-file
make
make install-strip
cd ..
rm -r install/lib/pkgconfig
ln -s ../bin/libgta-0.dll install/lib/libgta.dll

# Modify installation
VERSION="`grep " GTA_VERSION " install/include/gta/gta_version.h | sed -e 's/.*"\(.*\)"/\1/'`"
for i in AUTHORS COPYING NEWS README; do
	cp "$LIBGTASRC"/"$i" install/doc
	recode ../crlf install/doc/"$i"
done
cd "$LIBGTASRC"/..
git log --date=iso --stat libgta > "$TMPD/install/doc/ChangeLog"
cd "$TMPD"
recode ../crlf install/doc/ChangeLog
cat > install/README.txt <<EOF
This package contains libgta and its prerequisites libz, libbz2, and liblzma.

See http://www.nongnu.org/gta/ for more information on this software.

This package was produced on a GNU/Linux system using
1) The gcc $CROSSPREFIX cross-compiler package
2) The following library packages:
   http://zlib.net/zlib-1.2.5.tar.bz2
   http://www.bzip.org/1.0.6/bzip2-1.0.6.tar.gz
   http://tukaani.org/xz/xz-5.0.3.tar.gz
3) The script doc/build-package.sh
EOF
recode ../crlf install/README.txt
cp "$D/build.sh" install/doc/build-package.sh
find install -type f -exec chmod 644 {} \;
find install -type d -exec chmod 755 {} \;

# Make archive
cp -a install libgta-"$VERSION"-$CROSSSHORTNAME
zip -9 -v -r libgta-"$VERSION"-$CROSSSHORTNAME.zip libgta-"$VERSION"-$CROSSSHORTNAME
cp libgta-"$VERSION"-$CROSSSHORTNAME.zip "$D"
chmod 644 "$D"/libgta-"$VERSION"-$CROSSSHORTNAME.zip

# Cleanup
cd "$D"
echo "Work directory was $TMPD"
#rm -rf "$TMPD"
}


# Preparation
if [ ! -d "$LIBGTASRC" ]; then
	echo "Please edit this script and set the path to your libgta sources."
	exit 1
fi
if [ ! -f zlib-1.2.5.tar.bz2 ]; then
	wget http://zlib.net/zlib-1.2.5.tar.bz2
fi
if [ ! -f bzip2-1.0.6.tar.gz ]; then
	wget http://www.bzip.org/1.0.6/bzip2-1.0.6.tar.gz
fi
if [ ! -f xz-5.0.3.tar.gz ]; then
	wget http://tukaani.org/xz/xz-5.0.3.tar.gz
fi

# Cross compiler to use
build_package w32 i686-w64-mingw32
build_package w64 x86_64-w64-mingw32
